<?php

namespace Drupal\cmlexchange\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\file\Entity\File;
use Drupal\cmlexchange\Protocol\Cml;
use Drupal\cmlmigrations\Controller\MigrationsController;

/**
 * CmlFile.
 */
class ImportPipeline extends ControllerBase {

  /**
   * Save file (init function).
   */
  public function process($cml_id) {
    $result = FALSE;
    $cml = \Drupal::entityManager()->getStorage('cml')->load($cml_id);
    $this->cml = $cml;
    $this->cml_id = $cml_id;
    // UnZip.
    if ($this->unZip($cml) != '>new') {
      Cml::debug(__CLASS__, "process: unZIP $cml_id");
      return FALSE;
    }
    // Process Data.
    if ($this->processData($cml) != '>done') {
      Cml::debug(__CLASS__, "process: Data $cml_id");
      return FALSE;
    }
    Cml::debug(__CLASS__, "process: OK! $cml_id");
    $result = TRUE;
    // RETURN: ($result) ? 'success' : 'progress'.
    return $result;
  }

  /**
   * Un Zip.
   */
  public function unZip($cml) {
    $result = FALSE;
    $config = \Drupal::config('cmlexchange.settings');
    $dir = 'cml-files';
    if ($config->get('file-path')) {
      $dir = $config->get('file-path');
    }
    // Path to CML data.
    $path = \Drupal::service('file_system')->realpath("public://$dir");
    $cmldir = FileSave::cmlDir($cml);
    // Current CML directory.
    $datadir = "$path/$cmldir";
    if ($cml->field_cml_status->value == 'zip') {
      $xmlfiles = $cml->field_cml_xml->getValue();
      $proccess = TRUE;
      // STEP 1. (*.zip)
      // Unzip (if *.ZIP attached).
      foreach ($xmlfiles as $key => $value) {
        $file = File::load($value['target_id']);
        $uri = $file->getFileUri();
        // Check *.zip!
        if (substr($uri, -4) == '.zip') {
          $filename = $file->getFilename();
          $cmd = "cd $datadir && unzip $filename > unzip.log";
          // Exec Unzip & wrire log.
          $result .= shell_exec($cmd);
          // Remove zip file.
          file_delete_multiple([$file->id()]);
          unset($xmlfiles[$key]);
          // Set proccess FLAG.
          $proccess = FALSE;
          Cml::debug(__CLASS__, "STEP1 unZip: $uri");
        }
      }
      // STEP 2 (check log modify time).
      // UnZip can be too long, need unzip.log timestamp check.
      if ($proccess) {
        $log = FALSE;
        if (!empty($xmlfiles)) {
          foreach ($xmlfiles as $key => $value) {
            $file = File::load($value['target_id']);
            $uri = $file->getFileUri();
            if (substr($uri, -4) == '.log') {
              $log = TRUE;
            }
          }
        }
        if (!$log) {
          // Set proccess FLAG.
          $proccess = FALSE;
          // 5 sec form log was changed.
          $logpath = "$datadir/unzip.log";
          if (file_exists($logpath) && filemtime($logpath) < REQUEST_TIME + 5) {
            $proccess = TRUE;
            // Move import_files to `CML-data/import_files`.
            $cmd = "cp -Rf import_files/ $path/";
            shell_exec("cd $datadir && $cmd && rm -r import_files");
            $result .= $cmd;
          }
          Cml::debug(__CLASS__, "STEP2: unZip done, LOG=OK");
        }
      }
      // STEP 3 (scan & attach xml).
      if ($proccess) {
        $xml = FALSE;
        if (!empty($xmlfiles)) {
          foreach ($xmlfiles as $key => $value) {
            $file = File::load($value['target_id']);
            $uri = $file->getFileUri();
            if (substr($uri, -4) == '.xml') {
              $xml = TRUE;
            }
          }
        }
        if (!$xml) {
          // Scan directory for files.
          $result .= ">scan: $datadir\n";
          foreach (scandir($datadir) as $filename) {
            // Find files.
            if (strpos($filename, ".") > 3) {
              $uri = "public://{$dir}/{$cmldir}/$filename";
              $result .= "$uri\n";
              // Save file as file-entity.
              $file = File::create([
                'uri' => $uri,
                'uid' => \Drupal::currentUser()->id(),
                'filename' => $filename,
                'status' => FILE_STATUS_PERMANENT,
              ]);
              $file->save();
              if ($file->id()) {
                // Add file to CML.
                $xmlfiles[] = ['target_id' => $file->id()];
              }
            }
          }
          // Set skip FLAG.
          $proccess = FALSE;
          Cml::debug(__CLASS__, "STEP3" . json_encode($xmlfiles));
        }
      }
      // STEP 4.
      // Done, Update files.
      if ($proccess) {
        $log = FALSE;
        if (!empty($xmlfiles)) {
          foreach ($xmlfiles as $key => $value) {
            $file = File::load($value['target_id']);
            $uri = $file->getFileUri();
            if (substr($uri, -4) == '.log') {
              $log = $uri;
            }
          }
        }
        if ($log) {
          $logpath = \Drupal::service('file_system')->realpath($log);
          $logdata = file_get_contents($logpath);
          foreach (explode("\n", $logdata) as $line) {
            if ($img = strstr($line, "import_files/")) {
              $imgpath = explode('/', $img);
              $filename = array_pop($imgpath);
              $filepath = implode('/', $imgpath);
              $uri = "public://{$dir}/$filepath/$filename";
              $result .= "$uri\n";
              $file = File::create([
                'uri' => $uri,
                'uid' => \Drupal::currentUser()->id(),
                'filename' => $filename,
                'status' => FILE_STATUS_PERMANENT,
              ]);
              $existing_files = entity_load_multiple_by_properties('file', ['uri' => $uri]);
              if (count($existing_files)) {
                $existing = reset($existing_files);
                $fid = $existing->id();
                $file->fid = $fid;
                $file->setOriginalId($fid);
                $result .= "Exist: $fid >> $uri\n";
              }
              $file->save();
            }
          }
        }
        $cml->field_cml_status->setValue('new');
        Cml::debug(__CLASS__, "STEP4: Done, Update files, OK");
      }
      $cml->field_cml_xml->setValue($xmlfiles);
      $cml->save();
    }
    else {
      $result = ">new";
    }
    return $result;
  }

  /**
   * Process Data.
   */
  public function processData($cml) {
    $result = 'start';
    if (\Drupal::moduleHandler()->moduleExists('cmlmigrations')) {
      $migrations = MigrationsController::getMigrations();
      if ($migrations) {
        Cml::debug(__CLASS__, "processData: " . $cml->id());
        $result = $this->import($migrations);
      }
    }
    else {
      Cml::debug(__CLASS__, "SKIP cmlmigrations: " . $cml->id());
      $result = '>done';
    }
    return $result;
  }

  /**
   * Import.
   */
  private function import($migrations) {
    $result = 'progress';

    $config = \Drupal::configFactory()->getEditable('cmlexchange.settings');
    $id = \Drupal::config('cmlexchange.settings')->get('current-import');
    if (is_numeric($cmlid)) {
      $config->set('current-import', $cmlid)->save();
      $cml = $this->cml;
      $time = $cml->created->value;
      $status = $cml->field_cml_status->value;
      if ($status == 'new') {
        $config->set('current-import', $cmlid)->save();
        $cml->field_cml_status->setValue('progress');
        $cml->save();
        // Good.
        $command = "nohup drush mi --group=cml --update > /dev/null";
        // Bad.
        $command = "drush mi --group=cml --update > /dev/null 2>/dev/null &";
        if (!$debug) {
          exec($command);
        }
      }
      if ($status == 'progress') {
        $config->set('current-import', $cmlid)->save();
        $progress = 'Idle';
        foreach ($migrations as $key => $migration) {
          if ($migration['status'] != 'Idle') {
            $progress = $migration['status'];
          }
        }
        if ($progress != 'Idle') {
          // Proccess.
          // if too long.
          $min = $id = \Drupal::config('cmlexchange.settings')->get('import-time');
          empty($min) ?? $min = 60;
          if ($cml->changed->value + 60 * $min < REQUEST_TIME) {
            $date1 = format_date($cml->changed->value + 60 * $min, 'custom', "dM H:i:s");
            $date2 = format_date(REQUEST_TIME, 'custom', "dM H:i:s");
            \Drupal::logger('cmlexchange')->error("cml:$cmlid import timeout failure. $date1 < $date2");
            $config->set('current-import', FALSE)->save();
            $cml->field_cml_status->setValue('failure');
            // Old: $cml->save();.
            // Old : $result = 'failure';.
          }
        }
        else {
          Cml::debug(__CLASS__, "$cmlid: success");
          $config->set('current-import', FALSE)->save();
          $cml->field_cml_status->setValue('success');
          $cml->save();
        }
      }
      if ($status == 'success') {
        $result = 'success';
      }
      Cml::debug(__CLASS__, "$cmlid: $status: $result");
    }
    return $result;
  }

}
